#ifndef WBLREGION_H
#define WBLREGION_H

#include "dart.h"
#include "wClosures.h"
#include "wViscMesh.h"
#include "wInvBnd.h"
#include <vector>
#include <string>

namespace dart
{

class DART_API BLRegion
{

private:
    unsigned int const nVar = 5;

    /* Transition related variables */

    double nCrit = 9.0;


public:

    ViscMesh *mesh;
    InvBnd *invBnd;
    Closures *closSolver;

    std::string name;   /* Name of the region */

    /* Boundary layer variables */

    std::vector<double> U;      /* Solution vector (theta, H, N, ue, Ct)                          */
    std::vector<double> Ret;    /* Reynolds number based on the momentum thickness (theta)        */
    std::vector<double> Cd;     /* Local dissipation coefficient                                  */
    std::vector<double> Cf;     /* Local friction coefficient                                     */
    std::vector<double> Hstar;
    std::vector<double> Hstar2;
    std::vector<double> Hk;
    std::vector<double> Cteq;
    std::vector<double> Us;
    std::vector<double> Delta;

    std::vector<double> DeltaStar;

    /* Transition related variables */

    std::vector<int> Regime;
    std::vector<double> Blowing;

    unsigned int transMarker;
    double xtrF;
    double xtr;

    BLRegion(double _xtrF = -1.0);
    ~BLRegion();

    /* Distribute BC */
    void SetMesh(std::vector<double> x,
                 std::vector<double> y,
                 std::vector<double> z);
    
    /* Boundary conditions */
    void SetStagBC(double Re);
    void SetWakeBC(double Re, std::vector<double> UpperCond, std::vector<double> LowerCond);
    
    /* General access */
    unsigned int GetnVar() const {return nVar;};
    double GetnCrit() const {return nCrit;};

    /* Others */
    void PrintSolution(size_t iPoint) const;
    void ComputeBlwVel();

};
} // namespace dart
#endif //WBLREGION_H